/******************************************************************************
 * resecure - resecure file permissions after system upgrade
 * Copyright (C) 2000  Bill Marquette <billm@danger.ms>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *****************************************************************************/

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "dllist.h"
#include "resecure.h"



/******************************************************************************
 * Function: add_to_list
 * Purpose: add a node to the tail of the linked list
 *****************************************************************************/
ListPtr add_to_list(void *data, int sizeofdata, NodePtr node, ListPtr list)
{
  NodePtr new_rec = NULL;

  /* If first then we create the list management struct */
  if (list == NULL) list = (ListPtr)malloc(sizeof(List));

  new_rec = (NodePtr)malloc(sizeof(Node));
  if (!new_rec)
  {
    printf("\nUnable to allocate memory!\n");
    exit(1);
  }

  /* set new link's data */
  new_rec->data = (void *)malloc(sizeofdata);
  bcopy((void *)data, (void *)new_rec->data, (size_t)sizeofdata);
  new_rec->next = NULL;
  new_rec->prev = node; /* Set to "current" */

  if (node == NULL) /* Adding first link to list */
  {
    list->head = new_rec; /* Setup master pointer */
    list->tail = new_rec; /* Setup master pointer */
    list->current = new_rec;
    list->current_index = 1;
    list->listsize = 1;
    new_rec->next = NULL; /* Redundant but safe */
    new_rec->prev = NULL; /* Redundant but safe */
  }
  else
  {
    node->next = new_rec; /* Point "current" to new */
    list->tail = new_rec;
    list->current = new_rec;
    list->current_index++;
    list->listsize++;
    new_rec->prev = node; /* Redundant but safe */
    new_rec->next = NULL; /* Redundant but safe */
  }
  return list;
}

/******************************************************************************
 * Function: set_list_to_head
 * Purpose: Set current pointer back to head of list
 *****************************************************************************/
void set_list_to_head(ListPtr list)
{
  list->current = list->head;
  list->current_index = 1;
}


/******************************************************************************
 * Function: get_next_node
 * Purpose: Get the next node in the list
 *****************************************************************************/
ListPtr get_next_node(ListPtr list)
{
  if(list->current != list->tail)
  {
    list->current_index++;
    list->current = list->current->next;

    return list;
  } else 
    return NULL;
}


/******************************************************************************
 * Function: free_list
 * Purpose: free() the linked list - head to tail
 *****************************************************************************/
void free_list(ListPtr list)
{
  NodePtr cur_ptr;

  if (list == NULL) return;

  list->current = list->head; /* Set to head of list */

  cur_ptr = list->current;    /* initialize cur_ptr */
  list->head = cur_ptr->next; /* Set list head to next record */
  list->listsize--;           /* Shrink list counter by one */
  free(cur_ptr->data);        /* Free the data in the list */
  free(cur_ptr);              /* Free the current record */

  while ((list = get_next_node(list)) != NULL)
  {
    cur_ptr = list->current;    /* Set cur_ptr to beginning of list */
    list->head = cur_ptr->next; /* Set beginning of list to next record */
    list->listsize--;           /* Shrink list counter by one */
    free(cur_ptr->data);        /* Free the data in the list */
    free(cur_ptr);              /* Free the current record */
  }
  
  free(list); /* Free the list itself */

}
