/******************************************************************************
 * resecure - resecure file permissions after system upgrade
 * Copyright (C) 2000  Bill Marquette <billm@danger.ms>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *****************************************************************************/

#ifndef RESECURE_H
#define RESECURE_H

#include <sys/types.h>
#include <sys/stat.h>

/* man chmod(2) for the meanings of these bits */
/* User */
#define S_ISRUSR(m)         (m & S_IRUSR)
#define S_ISWUSR(m)         (m & S_IWUSR)
#define S_ISXUSR(m)         (m & S_IXUSR)

/* Group */
#define S_ISRGRP(m)         (m & S_IRGRP)
#define S_ISWGRP(m)         (m & S_IWGRP)
#define S_ISXGRP(m)         (m & S_IXGRP)

/* Other */
#define S_ISROTH(m)         (m & S_IROTH)
#define S_ISWOTH(m)         (m & S_IWOTH)
#define S_ISXOTH(m)         (m & S_IXOTH)

/* Other modes */
#define S_ISSUID(m)         (m & S_ISUID)
#define S_ISSGID(m)         (m & S_ISGID)
#define S_ISSVTX(m)         (m & S_ISVTX)

/* man chflags(2) for the meanings of these bits */
/* User */
#define S_ISUF_NODUMP(m)    (m & UF_NODUMP)
#define S_ISUF_IMMUTABLE(m) (m & UF_IMMUTABLE)
#define S_ISUF_APPEND(m)    (m & UF_APPEND)
#define S_ISUF_OPAQUE(m)    (m & UF_OPAQUE)

/* Super-user */
#define S_ISSF_ARCHIVED(m)  (m & SF_ARCHIVED)
#define S_ISSF_IMMUTABLE(m) (m & SF_IMMUTABLE)
#define S_ISSF_APPEND(m)    (m & SF_APPEND)

typedef struct {
  char filename[FILENAME_MAX+1]; /* Some day I'll shorten this */
  char dirname[FILENAME_MAX+1];  /* Ditto here */
  char hash[41];      /* File hash */
  struct stat sb;
} file_struct;

#ifndef RMDsize
#define RMDsize 160
#endif

#endif
